/** @file generate_key_pair.cpp
 * Command line utility for generationg RSA key pairs.
 */

/*
 * begin        : Mon Mar 10 2002
 * copyright    : (C) 2003 by Ondrej Pecta
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it according to instructions in massiv/doc/licenses/source.txt.       *
 *                                                                         *
 ***************************************************************************/

#include "global/common.h"
#include "utility/quote_string.h"
#include "net/massiv_rsa.h"
#include <iostream>
#include <sstream>
#include <fstream>
#include <time.h>

using namespace Massiv::Core;

std::string generate_seed
    (
    int len
    )
    {
    CryptoPP::RandomPool    pool;
    int                     i = 0;
    std::string             key;

    ::srand( static_cast< unsigned int >( ::time( NULL ) ) );
    pool.Put( rand() );

    for( i = 0; i < len ; i++ )
        key += pool.GenerateByte();

    return key;
    }

int main
    (
    int        argc,
    char **    argv
    )
    {
    unsigned int bitlen = 512;
    unsigned int seedlen = 64;

    if( argc < 2 )
        {
        std::cout << "Invalid argument count( " << argc - 1 << " )" << std::endl;
        std::cout << "Usage: " << argv[ 0 ] << " [ seedlen ]" << std::endl;
        std::cout << "Generate RSA key pair with specified bit length." << std::endl;
        std::cout << "Seedlen is the size of the RSA seed." << std::endl;
        std::cout << "Keys are saved as binary files rsa_priv.rsa and rsa_publ.rsa." << std::endl;
        std::cout << "Part of massiv config containing both public and private keys will be" << std::endl;
        std::cout << "saved to rsa_keys.conf." << std::endl;
        return( 1 );
        }
    std::string arg = argv[ 1 ];
    std::istringstream ss( arg );
    ss >> bitlen;

    if( argc > 2 )
        {
        std::string arg = argv[ 2 ];
        std::istringstream ss( arg );
        ss >> seedlen;
        }

    RSACipher * rsa = new RSACipher( bitlen, generate_seed( seedlen ) );
    std::string priv_key = rsa->get_private_key();
    std::string publ_key = rsa->get_public_key();

    // Raw private key.
    
    std::ofstream ofpriv( "rsa_priv.rsa", std::ofstream::binary );
    ofpriv << priv_key;
    ofpriv.close();

    // Raw public key.
    
    std::ofstream ofpubl( "rsa_publ.rsa", std::ofstream::binary );
    ofpubl << publ_key;
    ofpubl.close();

    // Quoted private and public keys.
    
    std::ofstream ofconf( "rsa_keys.conf", std::ofstream::binary );
    ofconf << "rsa_public_key : string = " << quote_string( publ_key, true ) << std::endl;
    ofconf << "rsa_private_key : string = " << quote_string( priv_key, true ) << std::endl;
    ofconf.close();

    return( 0 );
    }
